/* app.c
 *
 * Copyright (C) 2004 Xin Zhen
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <config.h>
#include "app.h"
//#include "menus.h"

#include <gtk/gtkmenu.h>
#include <glib/gi18n-lib.h>
#include "eggtrayicon.h"
#include "resources.h"
#include "sendfile.h"
#include "recvfile.h"
#include "irda_misc.h"

/* Keep a list of all open application windows */
static GSList* app_list = NULL;

static gint delete_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data);
static void button_click_cb(GtkWidget* w, gpointer data);

static void trayicon_init();
void discoverer_startpoll();
static GtkWidget * show_device_info_cb( void *data );
static GtkWidget * show_log_file( void *data );

GtkWidget* 
ircptray_app_new(const gchar* message, 
              const gchar* geometry,
              GSList* greet)
{
  GtkWidget* app = NULL;
  GtkWidget* button;
  GtkWidget* label;
  GtkWidget* status;
  GtkWidget* frame;

  /*** gnomehello-widgets */
  //app = gnome_app_new(PACKAGE, _("IRCP Tray"));

  frame = gtk_frame_new(NULL);

  //button = gtk_button_new();

  //label  = gtk_label_new(message ? message : _("Hello, World!"));

  //gtk_window_set_policy(GTK_WINDOW(app), TRUE, TRUE, FALSE);
  //gtk_window_set_default_size(GTK_WINDOW(app), 250, 350);
  //gtk_window_set_wmclass(GTK_WINDOW(app), "ircptray", "IRCP-Tray");

  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_frame_set_label(GTK_FRAME(frame), PACKAGE);

  //gtk_container_set_border_width(GTK_CONTAINER(button), 10);

  //gtk_container_add(GTK_CONTAINER(button), label);

  //gtk_container_add(GTK_CONTAINER(frame), button);

  //gnome_app_set_contents(GNOME_APP(app), frame);

  //status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);

  //gnome_app_set_statusbar(GNOME_APP(app), status);

  //hello_install_menus_and_toolbar(app);

  /* gnomehello-widgets ***/
  
  /*** gnomehello-signals */
  /*g_signal_connect(G_OBJECT(app),
                   "delete_event",
                   G_CALLBACK(delete_event_cb),
                   NULL);*/

  /*g_signal_connect(G_OBJECT(button),
                   "clicked",
                   G_CALLBACK(button_click_cb),
                   label);*/
                   
  /* gnomehello-signals ***/

  /* gnomehello-geometry ***/
  if (geometry != NULL) 
    {
      if (!gtk_window_parse_geometry (GTK_WINDOW(app), geometry)) 
        {
          g_error(_("Could not parse geometry string `%s'"), geometry);
        }
    }

  app_list = g_slist_prepend(app_list, app);

	//sanity check
	if(!irda_check()) //no irda support
	{
		GtkWidget* dialog;

		dialog = gtk_message_dialog_new(NULL,
                                        GTK_DIALOG_MODAL,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_CLOSE,
                                        _("You don't seem to have IrDA enabled in your kernel, quit."),
                                        NULL);
		gtk_dialog_run(GTK_DIALOG(dialog));
		return dialog;
	}

	trayicon_init();

	recvfile_listen(TRUE);
	
	discoverer_startpoll();

	return app;
}

//static EggTrayIcon* TrayIcon;
EggTrayIcon* TrayIcon;
static GtkImage* TrayIconImage;
static GtkWidget* IconPopMenuItem;

#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 12
#else
static GtkTooltips* TrayToolTips;
#endif

static int TrayIconAnimationRefCount=0;
static int TrayIconTimerId = 0;
static char* TrayIconFileName = ICON_TRAY_INACTIVE;

static gboolean trayicon_onclick(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data)
{
	if(event->button == 3) //right click
		gtk_menu_popup(GTK_MENU(user_data), NULL, NULL, NULL,
				NULL, event->button,
				gtk_get_current_event_time());
	else if(event->button == 1) //left click
		sendfile_launch_dialog();
}

static GtkTargetEntry target_table[] =
{
    {"text/uri-list",	0, 0},
    //{"STRING",		0, 1},
    //{"text/plain",	0, 2}
};

static const guint n_targets = sizeof(target_table) / sizeof(target_table[0]);

static void trayicon_init()
{
    GtkWidget* icon;
    GtkWidget* label;
    GtkWidget* menu, *menuitem;
    GtkWidget* image01, *image02;
    GtkWidget* eventbox;
    unsigned long rc;
                                                                            
    TrayIcon = egg_tray_icon_new("ircp-tray");
                                                                                
	//building icon popup menu
	menu = gtk_menu_new();
	//menuitem = gtk_image_menu_item_new();
	//label = gtk_label_new("");
	menuitem = gtk_image_menu_item_new_with_label(_("Transfer Files..."));
	image01 = gtk_image_new_from_stock (GTK_STOCK_GOTO_TOP, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(menuitem, image01);
	
	//gtk_label_set_markup(GTK_LABEL(label), _("<b>Transfer Files...</b>"));
	//gtk_container_add(GTK_CONTAINER(menuitem), label);
	g_signal_connect(G_OBJECT(menuitem),
			"activate",
			G_CALLBACK(sendfile_onclick),
			NULL);
	gtk_container_add(GTK_CONTAINER(menu), menuitem);
	
	image02 = gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU);
	IconPopMenuItem = menuitem = gtk_image_menu_item_new_with_label(_("Get Devices Info..."));
	gtk_image_menu_item_set_image(menuitem, image02);
	gtk_container_add(GTK_CONTAINER(menu), menuitem);
	g_signal_connect(G_OBJECT(menuitem),
                        "activate",
                        G_CALLBACK(show_device_info_cb),
                        NULL);
	gtk_widget_set_sensitive(menuitem, FALSE);

	menuitem = gtk_menu_item_new_with_label(_("Show History..."));
	gtk_container_add(GTK_CONTAINER(menu), menuitem);
        g_signal_connect(G_OBJECT(menuitem),
                        "activate",
                        G_CALLBACK(show_log_file),
                        NULL);

	menuitem = gtk_separator_menu_item_new();
	gtk_container_add(GTK_CONTAINER(menu), menuitem);

	menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
	//menuitem = gtk_menu_item_new_with_label(_("About"));
	
	gtk_container_add(GTK_CONTAINER(menu), menuitem);
	g_signal_connect(G_OBJECT(menuitem),
			"activate",
			G_CALLBACK(version_onclick),
			NULL);
	menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
	/*menuitem = gtk_menu_item_new_with_label(_("Quit"));*/
	gtk_container_add(GTK_CONTAINER(menu), menuitem);
	g_signal_connect(G_OBJECT(menuitem),
			"activate",
			G_CALLBACK(gtk_main_quit),
			NULL);

	gtk_widget_show_all(menu);

	//icon = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
	TrayIconImage = (GtkImage* )gtk_image_new_from_icon_name(TrayIconFileName, GTK_ICON_SIZE_SMALL_TOOLBAR);
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(eventbox), GTK_WIDGET(TrayIconImage));
	g_signal_connect(G_OBJECT(eventbox),
                   "button-press-event",
                   G_CALLBACK(trayicon_onclick),
                   menu);

	gtk_container_add(GTK_CONTAINER(TrayIcon), eventbox);

	//d'n'd
	gtk_drag_dest_set(
			GTK_WIDGET(TrayIcon),
			GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
			target_table,
			n_targets,
			GDK_ACTION_COPY
		);

	//tooltips

#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 12
#else
	TrayToolTips = gtk_tooltips_new               ();
#endif


#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 12
	gtk_widget_set_tooltip_text (GTK_WIDGET(TrayIcon),  _("Infrared File Transfer"));
#else
	gtk_tooltips_set_tip(TrayToolTips, GTK_WIDGET(TrayIcon), _("Infrared File Transfer"), "ircp-tray");
#endif

	g_signal_connect(G_OBJECT(TrayIcon), "drag_data_received",
				G_CALLBACK(sendfile_ondragdrop), NULL);
	
	gtk_widget_show_all(GTK_WIDGET(TrayIcon));
}

void trayicon_set_text(char* str)
{
	int rc = egg_tray_icon_send_message(TrayIcon, 5, str, strlen(str));
}

void trayicon_set_icon(char* filename)
{
	TrayIconFileName = filename;
	gtk_image_set_from_icon_name(TrayIconImage, TrayIconFileName, GTK_ICON_SIZE_SMALL_TOOLBAR);
}

gboolean  trayicon_animation_ontimer (gpointer data)
{
	char* animtable[] = {
		ICON_TRAY_ANIM1,
		ICON_TRAY_ANIM2,
		ICON_TRAY_ANIM3,
		ICON_TRAY_ANIM4
	};
	static animindex = 0;
	int animnum = sizeof(animtable)/sizeof(char*);
	
	gtk_image_set_from_icon_name(TrayIconImage, animtable[animindex], GTK_ICON_SIZE_SMALL_TOOLBAR);
	
	animindex = (animindex+1) % animnum;
	
	return TRUE;
}

void trayicon_animation_start()
{
	if(TrayIconAnimationRefCount == 0) {//setup a timer
		TrayIconTimerId = g_timeout_add_full(
							G_PRIORITY_LOW,
							1000, //1 second
							trayicon_animation_ontimer,
							NULL, NULL);
	}
	TrayIconAnimationRefCount++;
	printf("animation start %d\n", TrayIconAnimationRefCount);
}

void trayicon_animation_stop()
{
	TrayIconAnimationRefCount--;
	printf("animation stop %d\n", TrayIconAnimationRefCount);
	
	if(TrayIconAnimationRefCount == 0) {//destroy timer
		g_source_remove(TrayIconTimerId);
		gtk_image_set_from_icon_name(TrayIconImage, TrayIconFileName, GTK_ICON_SIZE_MENU);//restore default
	}
}

void version_onclick(GtkMenuItem *menuitem, gpointer user_data)
{
	struct SaveFileDialog* dialog;

	char* authors[] = {
		"Xin Zhen <xlonestar2000@aim.com>",
		"",
		"Marcel Holtmann",
		"Pontus Fuchs",
		"Ragnar Henriksen",
		NULL
	};

	char* artists[] = {
		"Icon design: Jakub Steiner <jimmac@ximian.com>",
		"Animation: Dieter Vanderfaeillie <dieter.vanderfaeillie@gmail.com>",
		NULL
	};

	char* translators = 
		"Polish: Adrian Łubik <adrian5632@gmail.com>\n"
		"Swedish: Daniel Nylander <info@danielnylander.se>";
	
	/*{
		struct _dev_info *devices;
        struct _dev_info *p;
		int numdev, i;
		devices = irda_get_hints( &numdev );
        if ( numdev <= 0 )
                printf("none remote device found\n");
		p = devices;
        i = 0;
        while ( p ) {
			printf("name: %s, hint0 %x, hint1 %x\n",
					p->name, p->hints[0], p->hints[1]);
			p = p->next;
		}
		
		free_dev_info(devices);
	}*/
#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 6
	gtk_show_about_dialog(GTK_WINDOW(TrayIcon),
                          "authors", authors,
			  "artists", artists,
			  "translator-credits", translators,
                          "comments", _("A utility for IrDA OBEX beaming and receiving"),
                          "copyright", "Xin Zhen (LoneStar) <xlonestar2000@aim.com>",
                          "version", VERSION,
                          "website", "http://www.gnome-cn.org/software/ircp-tray",
			  "logo-icon-name", "ircp-tray",
                          NULL);
#endif
	/*dialog = savefile_dialog_new("About Ircp-Tray");
	savefile_dialog_run (dialog);
	savefile_dialog_destroy (dialog);*/
}

char* config_get_lastsavedir()
{
		char* lastsavedir = NULL;
		char* configfilename;

#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 6
		struct GKeyFile* config;

		configfilename = malloc(strlen(g_get_home_dir())+11);
		sprintf(configfilename, "%s/.ircp-tray", g_get_home_dir());

		config = g_key_file_new();
		g_key_file_load_from_file(config, configfilename,
			G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
		lastsavedir = g_key_file_get_string(config, "General", "LastSaveDir",
						NULL);
		g_key_file_free(config);
		free(configfilename);
#endif
		return lastsavedir;
}

void config_set_lastsavedir(char* dir)
{
		char* configfilename;
#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 6
		struct GKeyFile* config;
		char* configstr;
		FILE *configfile;
		
		configfilename = malloc(strlen(g_get_home_dir())+11);
		sprintf(configfilename, "%s/.ircp-tray", g_get_home_dir());

		config = g_key_file_new();
		g_key_file_load_from_file(config, configfilename,
			G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, NULL);
		g_key_file_set_string(config, "General", "LastSaveDir", dir);
		configstr = g_key_file_to_data(config, NULL, NULL);
		
		configfile = fopen(configfilename, "w");
		fprintf(configfile, configstr);
		fclose(configfile);
		
		free(configstr);
		free(configfilename);
		g_key_file_free(config);
#endif
}

void log_file(const char* message)
{
	char* logfile;
	FILE* fp;
	char filename[] = "/.ircp-tray-log";
        char tmstr[16];
        time_t tm = time(NULL);

	printf("%s\n", message);
        strftime(tmstr, 16, "%X", localtime(&tm));

	logfile = malloc(strlen(g_get_home_dir())+strlen(filename)+1);
	sprintf(logfile, "%s%s", g_get_home_dir(), filename);
	fp = fopen(logfile, "a");
	if(fp) {
		fprintf(fp, "%s: %s\n", tmstr, message);
		fclose(fp);
	}
	free(logfile);
}

static int discoverer_poll_timer;
static const int discoverer_poll_interval = 3000; //in milliseconds
#define discoverer_name_len 256
static char discoverer_name[discoverer_name_len];
static int discoverer_hints[2];

static gboolean discoverer_do(void*p)
{
	struct irda_device_list* list;
	char tmstr[16];
	time_t tm = time(NULL);

	strftime(tmstr, 16, "%X", localtime(&tm));	
	list = irda_discover_device(-1);
	if(list && list->len) {
		if(strncmp(discoverer_name, list->dev[0].info,
				discoverer_name_len) != 0) { //new device discovered
			char tooltips[discoverer_name_len+32];
			sprintf(tooltips, _("%s: Remote device discovered: %s"),
					 tmstr, list->dev[0].info);
			printf("%s\n", tooltips);
			trayicon_set_text(tooltips);
			trayicon_set_icon(ICON_TRAY);
			strncpy(discoverer_name, list->dev[0].info,
						discoverer_name_len);
			discoverer_name[discoverer_name_len-1] = '\0';
			discoverer_hints[0] = list->dev[0].hints[0];
			discoverer_hints[1] = list->dev[0].hints[1];
			sprintf(tooltips, _("Infrared File Transfer: %s"), discoverer_name);


#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 12
        gtk_widget_set_tooltip_text (GTK_WIDGET(TrayIcon),  tooltips);
#else
	gtk_tooltips_set_tip(TrayToolTips, GTK_WIDGET(TrayIcon), tooltips, "ircp-tray");
#endif


		        gtk_widget_set_sensitive(IconPopMenuItem, TRUE);
		}
	} else
	{
		if(strlen(discoverer_name)) {
			char byemsg[128];
			sprintf(byemsg, _("%s: Remote device %s disappeared"),
					 tmstr, discoverer_name);
			printf("%s\n", byemsg);
			trayicon_set_text(byemsg);
			discoverer_name[0] = '\0';
			trayicon_set_icon(ICON_TRAY_INACTIVE);

#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 12
        gtk_widget_set_tooltip_text (GTK_WIDGET(TrayIcon),  _("Infrared File Transfer"));
#else
        gtk_tooltips_set_tip(TrayToolTips, GTK_WIDGET(TrayIcon), _("Infrared File Transfer"), "ircp-tray");
#endif


                        gtk_widget_set_sensitive(IconPopMenuItem, FALSE);
		}
	}
	free(list);
}

void discoverer_startpoll()
{
	printf("start polling of devices in range, interval %d milliseconds\n",
				discoverer_poll_interval);
	discoverer_name[0] = '\0';
	discoverer_poll_timer = g_timeout_add(discoverer_poll_interval, discoverer_do, 0);
}

char* discoverer_get_name()
{
	return discoverer_name;
}

int* discoverer_get_hints()
{
	return discoverer_hints;
}

int discoverer_is_in_range()
{
	return strlen(discoverer_get_name()) > 0;
}

static void
on_dialog_response (GtkDialog *dialog, gint response, gpointer ok_handler_data)
{
	if (response == GTK_RESPONSE_CLOSE) {
		gtk_widget_destroy (GTK_WIDGET (dialog));
		dialog = 0;
		printf("on_dialog_response\n");
	}
}

static GtkWidget *
show_device_info_cb( void *data )
{
	GtkWidget *info;
	GtkWidget *vbox1;
	GtkWidget *close;
	GtkWidget *table1;
	GtkWidget *label1;
	GtkWidget *ck1;
	GtkWidget *aa1;
	gint i;
	gint y;
	gint numdev;
	struct _dev_info *devices;
	struct _dev_info *p;
	gchar *labels[] = {_("Hints"),
			   _("PNP"),
			   _("PDA"),
			   _("COMPUTER"),
			   _("PRINTER"),
			   _("MODEM"),
			   _("FAX"),
			   _("LAN"),
			   _("EXTENSION"),
			   _("TELEPHONY"),
			   _("FILESERVER"),
			   _("COMM"),
			   _("MESSAGE"),
			   _("HTTP"),
			   _("OBEX"),
			   NULL
	};

	devices = irda_get_hints( &numdev );
	printf("numdev %d\n", numdev);
	if ( numdev <= 0 )
		return 0;

	info = gtk_dialog_new ();
	//data->info_window = info;
	gtk_window_set_title (GTK_WINDOW (info), _("info"));
	//gtk_window_set_resizable (GTK_WINDOW (info), FALSE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (info), TRUE);
	gtk_dialog_set_default_response (GTK_DIALOG (info), GTK_RESPONSE_CLOSE);
 


	vbox1 =  GTK_DIALOG (info)->vbox;
	gtk_widget_show (vbox1);
	
	table1 = gtk_table_new (15, numdev+1, TRUE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

	/* Create labels */
	i = 0;
	while ( labels[i] ) {
		label1 = gtk_label_new (_(labels[i]));
		gtk_widget_show (label1);
		gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, i, i+1,
				  (GtkAttachOptions) (GTK_FILL),
				  (GtkAttachOptions) (0), 5, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
		i++;
	}

	/* Adding checkboxes */
	p = devices;
	i = 0;
	while ( p ) {
		label1 = gtk_label_new( p->name );
		gtk_widget_show (label1);
		gtk_table_attach (GTK_TABLE (table1), label1, i+1, i+2, 0, 1,
				  (GtkAttachOptions) (GTK_FILL),
				  (GtkAttachOptions) (0), 5, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
		
		y = 1;
		while ( labels[y] ) {
			ck1 = gtk_check_button_new();
			GTK_WIDGET_UNSET_FLAGS (ck1, GTK_CAN_FOCUS);
			gtk_widget_set_sensitive (ck1, FALSE);

			if ( (y <= 8) && (p->hints[0] & 1<<(y-1)) )
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ck1), 1 );

			if ( (y > 7) && (p->hints[1] & 1<<(y-9)) )
				gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(ck1), 1 );

			gtk_widget_show (ck1);
			gtk_table_attach (GTK_TABLE (table1), ck1, i+1, i+2, y, y+1,
					  (GtkAttachOptions) (GTK_FILL),
					  (GtkAttachOptions) (0), 5, 0);
			y++;
		}
		p = p->next;
		i++;
	}

	/*p = devices;
	while ( p ) {
		devices = p->next;
		free( p );
		p = devices;
	}*/
	free_dev_info(devices);

	/* Create close button */
	aa1 = GTK_DIALOG (info)->action_area;
	gtk_widget_show (aa1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (aa1), GTK_BUTTONBOX_END);
	
	close = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (close);
	gtk_dialog_add_action_widget (GTK_DIALOG (info), close, GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (close, GTK_CAN_DEFAULT);

	g_signal_connect (GTK_OBJECT (info), "response",
			    GTK_SIGNAL_FUNC (on_dialog_response),
			    NULL);

	gtk_widget_show(info);
	
	return info;
}

static GtkWidget * show_log_file( void *data )
{
	GtkWidget* window, *control;
	GtkTextBuffer* text;
	char* logfile;
	char* content;
	FILE* fp;
	int len;

	char filename[] = "/.ircp-tray-log";
        logfile = malloc(strlen(g_get_home_dir())+strlen(filename)+1);
        sprintf(logfile, "%s%s", g_get_home_dir(), filename);
	content = malloc(64 * 1024);
	fp = fopen(logfile, "r");
	if (fp)
		len = fread(content, 1, 64 * 1024, fp);
	else
		len = strlen(strcpy(content, "\n\tNone\t\n"));

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	text = gtk_text_buffer_new(NULL);
	gtk_text_buffer_set_text(text, content, len);
	control = gtk_text_view_new_with_buffer(text);
	gtk_container_add(GTK_CONTAINER(window), control);
	gtk_widget_show_all(window);

	free(content);
	if (fp)
		fclose(fp);
	free(logfile);
}

