/* app.c
 *
 * Copyright (C) 2004 Xin Zhen
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <savefiledlg.h>
#include <glib/gi18n.h>

struct SaveFileDialog* savefile_dialog_new(char* titlebar, char* devicename)
{
	struct SaveFileDialog* dialog = malloc(sizeof(struct SaveFileDialog));
	GtkWidget *widget, *box;
	char* path;

	/*dialog->dialogbox = gtk_dialog_new_with_buttons ("Incoming File Transfer Request",
                                         NULL,
                                         GTK_DIALOG_MODAL,
                                         GTK_STOCK_OK,GTK_RESPONSE_OK,
                                         NULL);*/
	dialog->dialogbox = gtk_message_dialog_new(NULL,
					GTK_DIALOG_MODAL,
					GTK_MESSAGE_QUESTION,
					GTK_BUTTONS_OK_CANCEL,
					_("Incoming File Transfer Request"));

	gtk_window_set_title(GTK_WINDOW(dialog->dialogbox), titlebar);
	//gtk_container_set_border_width (GTK_CONTAINER (dialog->dialogbox), 10);
	//gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog->dialogbox)->vbox), 10);


	//widget = gtk_label_new("A remote IrDA device is attempting to send you a file. Do you want to accept it?");
	//gtk_label_set_line_wrap(widget, TRUE);
	//gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog->dialogbox)->vbox),
        //              widget);

	gtk_dialog_set_default_response(GTK_DIALOG(dialog->dialogbox),
						GTK_RESPONSE_OK);
	gtk_message_dialog_format_secondary_text(
			GTK_MESSAGE_DIALOG(dialog->dialogbox),
			_("A remote IrDA device (%s) is attempting to send you a file. Do you want to accept it?"), devicename);

	//box 1
	box = gtk_hbox_new(FALSE, 12);

	//label
	widget = gtk_label_new(_("File name:"));
	gtk_box_pack_start(GTK_BOX(box), widget, FALSE, FALSE, 10);

	//entry
	dialog->currentname = widget = gtk_entry_new();
	gtk_entry_set_activates_default(GTK_ENTRY(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(box), widget, FALSE, FALSE, 0);

	//save to button
	dialog->filepath = widget = gtk_file_chooser_button_new(_("Save to"),
				GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
	gtk_box_pack_start(GTK_BOX(box), widget, TRUE, FALSE, 0);

	//gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog->dialogbox)->vbox),
        //              box);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog->dialogbox)->vbox),
			box, TRUE, FALSE, 10);

	path = config_get_lastsavedir();
	if(path) {
		savefile_dialog_set_current_folder(dialog, path);
		free(path);
	}
	return dialog;
}

void savefile_dialog_destroy(struct SaveFileDialog* dialog)
{
	char* path;
	path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog->filepath));
	if (dialog->respcode == GTK_RESPONSE_OK)
		config_set_lastsavedir(path);
	free(path);
	gtk_widget_destroy(dialog->dialogbox);

	free(dialog);
}

int savefile_dialog_run(struct SaveFileDialog* dialog)
{
	gtk_widget_show_all (dialog->dialogbox);
	return dialog->respcode = gtk_dialog_run(GTK_DIALOG(dialog->dialogbox));
}

//filename in utf8
void savefile_dialog_set_current_name(struct SaveFileDialog* dialog,
					char* filename_u)
{
	gtk_entry_set_text(GTK_ENTRY(dialog->currentname), filename_u);
}

gboolean savefile_dialog_set_current_folder(struct SaveFileDialog* dialog,
					char* filepath)
{
	return gtk_file_chooser_set_current_folder (
		GTK_FILE_CHOOSER(dialog->filepath), filepath);
}

char* savefile_dialog_get_filename(struct SaveFileDialog* dialog)
{
	char* pathname;
	const char*filename_u;
	char* filename;
	pathname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog->filepath));
	filename_u = gtk_entry_get_text(GTK_ENTRY(dialog->currentname));
	filename = g_filename_from_utf8(filename_u, -1, NULL, NULL, NULL);
	pathname = realloc(pathname, strlen(pathname)+strlen(filename)+2);
	strcat(pathname, "/");
	strcat(pathname, filename);
	free(filename);
	return pathname;
}
